@@ -21,9 +21,7 @@ import java.net.InetAddress;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
-import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Locale;
 
 import javax.management.ObjectName;
 
@@ -32,21 +30,26 @@ import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Service;
 import org.apache.catalina.core.AprLifecycleListener;
+import org.apache.catalina.core.AprStatus;
 import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.coyote.AbstractProtocol;
 import org.apache.coyote.Adapter;
 import org.apache.coyote.ProtocolHandler;
 import org.apache.coyote.UpgradeProtocol;
 import org.apache.coyote.http11.AbstractHttp11JsseProtocol;
+import org.apache.coyote.http11.AbstractHttp11Protocol;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.CharsetUtil;
 import org.apache.tomcat.util.buf.EncodedSolidusHandling;
+import org.apache.tomcat.util.buf.StringUtils;
 import org.apache.tomcat.util.buf.UDecoder;
+import org.apache.tomcat.util.compat.JreCompat;
 import org.apache.tomcat.util.net.SSLHostConfig;
 import org.apache.tomcat.util.net.openssl.OpenSSLImplementation;
+import org.apache.tomcat.util.net.openssl.OpenSSLStatus;
 import org.apache.tomcat.util.res.StringManager;
 
 
@@ -64,8 +67,8 @@ public class Connector extends LifecycleMBeanBase {
     /**
      * Alternate flag to enable recycling of facades.
      */
-    public static final boolean RECYCLE_FACADES = Boolean
-            .parseBoolean(System.getProperty("org.apache.catalina.connector.RECYCLE_FACADES", "false"));
+    public static final boolean RECYCLE_FACADES =
+            Boolean.parseBoolean(System.getProperty("org.apache.catalina.connector.RECYCLE_FACADES", "true"));
 
 
     public static final String INTERNAL_EXECUTOR_NAME = "Internal";
@@ -73,35 +76,40 @@ public class Connector extends LifecycleMBeanBase {
 
     // ------------------------------------------------------------ Constructor
 
+    /**
+     * Defaults to using HTTP/1.1 NIO implementation.
+     */
     public Connector() {
-        this(null);
+        this("HTTP/1.1");
     }
 
 
     public Connector(String protocol) {
-        setProtocol(protocol);
-        // Instantiate protocol handler
+        boolean apr = AprStatus.getUseAprConnector() && AprStatus.isInstanceCreated() &&
+                AprLifecycleListener.isAprAvailable();
         ProtocolHandler p = null;
         try {
-            Class<?> clazz = Class.forName(protocolHandlerClassName);
-            p = (ProtocolHandler) clazz.getConstructor().newInstance();
+            p = ProtocolHandler.create(protocol, apr);
         } catch (Exception e) {
             log.error(sm.getString("coyoteConnector.protocolHandlerInstantiationFailed"), e);
-        } finally {
-            this.protocolHandler = p;
         }
-
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
-            uriCharset = StandardCharsets.ISO_8859_1;
+        if (p != null) {
+            protocolHandler = p;
+            protocolHandlerClassName = protocolHandler.getClass().getName();
         } else {
-            uriCharset = StandardCharsets.UTF_8;
+            protocolHandler = null;
+            protocolHandlerClassName = protocol;
         }
+        // Default for Connector depends on this system property
+        setThrowOnFailure(Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"));
+    }
 
-        // Default for Connector depends on this (deprecated) system property
-        if (Boolean
-                .parseBoolean(System.getProperty("org.apache.tomcat.util.buf.UDecoder.ALLOW_ENCODED_SLASH", "false"))) {
-            encodedSolidusHandling = EncodedSolidusHandling.DECODE;
-        }
+
+    public Connector(ProtocolHandler protocolHandler) {
+        protocolHandlerClassName = protocolHandler.getClass().getName();
+        this.protocolHandler = protocolHandler;
+        // Default for Connector depends on this system property
+        setThrowOnFailure(Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"));
     }
 
 
@@ -137,12 +145,6 @@ public class Connector extends LifecycleMBeanBase {
     protected boolean xpoweredBy = false;
 
 
-    /**
-     * The port number on which we listen for requests.
-     */
-    protected int port = -1;
-
-
     /**
      * The server name to which we should pretend requests to this Connector were directed. This is useful when
      * operating Tomcat behind a proxy server, so that redirects get constructed accurately. If not specified, the
@@ -198,18 +200,18 @@ public class Connector extends LifecycleMBeanBase {
 
     /**
      * The maximum number of parameters (GET plus POST) which will be automatically parsed by the container. 10000 by
-     * default. A value of less than 0 means no limit.
+     * default. The default Tomcat server.xml configures a lower default of 1000. A value of less than 0 means no limit.
      */
     protected int maxParameterCount = 10000;
 
     /**
-     * Maximum size of a POST which will be automatically parsed by the container. 2MB by default.
+     * Maximum size of a POST which will be automatically parsed by the container. 2 MiB by default.
      */
     protected int maxPostSize = 2 * 1024 * 1024;
 
 
     /**
-     * Maximum size of a POST which will be saved by the container during authentication. 4kB by default
+     * Maximum size of a POST which will be saved by the container during authentication. 4 KiB by default
      */
     protected int maxSavePostSize = 4 * 1024;
 
@@ -232,9 +234,9 @@ public class Connector extends LifecycleMBeanBase {
 
 
     /**
-     * Coyote Protocol handler class name. Defaults to the Coyote HTTP/1.1 protocolHandler.
+     * Coyote Protocol handler class name. See {@link #Connector()} for current default.
      */
-    protected String protocolHandlerClassName = "org.apache.coyote.http11.Http11NioProtocol";
+    protected final String protocolHandlerClassName;
 
 
     /**
@@ -249,22 +251,6 @@ public class Connector extends LifecycleMBeanBase {
     protected Adapter adapter = null;
 
 
-    /**
-     * URI encoding.
-     *
-     * @deprecated This will be removed in 9.0.x onwards
-     */
-    @Deprecated
-    protected String URIEncoding = null;
-
-
-    /**
-     * @deprecated This will be removed in 9.0.x onwards
-     */
-    @Deprecated
-    protected String URIEncodingLower = null;
-
-
     /**
      * The URI encoding in use.
      */
@@ -275,8 +261,8 @@ public class Connector extends LifecycleMBeanBase {
      * The behavior when an encoded solidus (slash) is submitted.
      */
     @SuppressWarnings("deprecation")
-    private EncodedSolidusHandling encodedSolidusHandling = UDecoder.ALLOW_ENCODED_SLASH ? EncodedSolidusHandling.DECODE
-            : EncodedSolidusHandling.REJECT;
+    private EncodedSolidusHandling encodedSolidusHandling =
+            UDecoder.ALLOW_ENCODED_SLASH ? EncodedSolidusHandling.DECODE : EncodedSolidusHandling.REJECT;
 
 
     /**
@@ -285,15 +271,6 @@ public class Connector extends LifecycleMBeanBase {
     protected boolean useBodyEncodingForURI = false;
 
 
-    protected static final HashMap<String, String> replacements = new HashMap<>();
-    static {
-        replacements.put("acceptCount", "backlog");
-        replacements.put("connectionLinger", "soLinger");
-        replacements.put("connectionTimeout", "soTimeout");
-        replacements.put("rootFile", "rootfile");
-    }
-
-
     // ------------------------------------------------------------- Properties
 
     /**
@@ -304,11 +281,10 @@ public class Connector extends LifecycleMBeanBase {
      * @return the property value
      */
     public Object getProperty(String name) {
-        String repl = name;
-        if (replacements.get(name) != null) {
-            repl = replacements.get(name);
+        if (protocolHandler == null) {
+            return null;
         }
-        return IntrospectionUtils.getProperty(protocolHandler, repl);
+        return IntrospectionUtils.getProperty(protocolHandler, name);
     }
 
 
@@ -321,11 +297,10 @@ public class Connector extends LifecycleMBeanBase {
      * @return <code>true</code> if the property was successfully set
      */
     public boolean setProperty(String name, String value) {
-        String repl = name;
-        if (replacements.get(name) != null) {
-            repl = replacements.get(name);
+        if (protocolHandler == null) {
+            return false;
         }
-        return IntrospectionUtils.setProperty(protocolHandler, repl, value);
+        return IntrospectionUtils.setProperty(protocolHandler, name, value);
     }
 
 
@@ -539,7 +514,7 @@ public class Connector extends LifecycleMBeanBase {
         HashSet<String> methodSet = new HashSet<>();
 
         if (null != methods) {
-            methodSet.addAll(Arrays.asList(methods.split("\\s*,\\s*")));
+            methodSet.addAll(Arrays.asList(StringUtils.splitCommaSeparated(methods)));
         }
 
         if (methodSet.contains("TRACE")) {
@@ -562,7 +537,18 @@ public class Connector extends LifecycleMBeanBase {
      *             means select a random free port when the socket is bound.
      */
     public int getPort() {
-        return this.port;
+        // Try shortcut that should work for nearly all uses first as it does
+        // not use reflection and is therefore faster.
+        if (protocolHandler instanceof AbstractProtocol<?>) {
+            return ((AbstractProtocol<?>) protocolHandler).getPort();
+        }
+        // Fall back for custom protocol handlers not based on AbstractProtocol
+        Object port = getProperty("port");
+        if (port instanceof Integer) {
+            return ((Integer) port).intValue();
+        }
+        // Usually means an invalid protocol has been configured
+        return -1;
     }
 
 
@@ -572,7 +558,6 @@ public class Connector extends LifecycleMBeanBase {
      * @param port The new port number
      */
     public void setPort(int port) {
-        this.port = port;
         setProperty("port", String.valueOf(port));
     }
 
@@ -620,49 +605,17 @@ public class Connector extends LifecycleMBeanBase {
     /**
      * @return the Coyote protocol handler in use.
      */
+    @SuppressWarnings("deprecation")
     public String getProtocol() {
-        if (("org.apache.coyote.http11.Http11NioProtocol".equals(getProtocolHandlerClassName()) &&
-                (!AprLifecycleListener.isAprAvailable() || !AprLifecycleListener.getUseAprConnector())) ||
-                "org.apache.coyote.http11.Http11AprProtocol".equals(getProtocolHandlerClassName()) &&
-                        AprLifecycleListener.getUseAprConnector()) {
+        boolean apr = AprStatus.getUseAprConnector();
+        if ((!apr && org.apache.coyote.http11.Http11NioProtocol.class.getName().equals(protocolHandlerClassName)) ||
+                (apr && org.apache.coyote.http11.Http11AprProtocol.class.getName().equals(protocolHandlerClassName))) {
             return "HTTP/1.1";
-        } else if (("org.apache.coyote.ajp.AjpNioProtocol".equals(getProtocolHandlerClassName()) &&
-                (!AprLifecycleListener.isAprAvailable() || !AprLifecycleListener.getUseAprConnector())) ||
-                "org.apache.coyote.ajp.AjpAprProtocol".equals(getProtocolHandlerClassName()) &&
-                        AprLifecycleListener.getUseAprConnector()) {
+        } else if ((!apr && org.apache.coyote.ajp.AjpNioProtocol.class.getName().equals(protocolHandlerClassName)) ||
+                (apr && org.apache.coyote.ajp.AjpAprProtocol.class.getName().equals(protocolHandlerClassName))) {
             return "AJP/1.3";
         }
-        return getProtocolHandlerClassName();
-    }
-
-
-    /**
-     * Set the Coyote protocol which will be used by the connector.
-     *
-     * @param protocol The Coyote protocol name
-     *
-     * @deprecated Will be removed in Tomcat 9. Protocol must be configured via the constructor
-     */
-    @Deprecated
-    public void setProtocol(String protocol) {
-
-        boolean aprConnector = AprLifecycleListener.isAprAvailable() && AprLifecycleListener.getUseAprConnector();
-
-        if ("HTTP/1.1".equals(protocol) || protocol == null) {
-            if (aprConnector) {
-                setProtocolHandlerClassName("org.apache.coyote.http11.Http11AprProtocol");
-            } else {
-                setProtocolHandlerClassName("org.apache.coyote.http11.Http11NioProtocol");
-            }
-        } else if ("AJP/1.3".equals(protocol)) {
-            if (aprConnector) {
-                setProtocolHandlerClassName("org.apache.coyote.ajp.AjpAprProtocol");
-            } else {
-                setProtocolHandlerClassName("org.apache.coyote.ajp.AjpNioProtocol");
-            }
-        } else {
-            setProtocolHandlerClassName(protocol);
-        }
+        return protocolHandlerClassName;
     }
 
 
@@ -674,19 +627,6 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    /**
-     * Set the class name of the Coyote protocol handler which will be used by the connector.
-     *
-     * @param protocolHandlerClassName The new class name
-     *
-     * @deprecated Will be removed in Tomcat 9. Protocol must be configured via the constructor
-     */
-    @Deprecated
-    public void setProtocolHandlerClassName(String protocolHandlerClassName) {
-        this.protocolHandlerClassName = protocolHandlerClassName;
-    }
-
-
     /**
      * @return the protocol handler associated with the connector.
      */
@@ -809,17 +749,6 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    /**
-     * @return the character encoding to be used for the URI using lower case.
-     *
-     * @deprecated This will be removed in 9.0.x onwards
-     */
-    @Deprecated
-    public String getURIEncodingLower() {
-        return uriCharset.name().toLowerCase(Locale.ENGLISH);
-    }
-
-
     /**
      * @return The Charset to use to convert raw URI bytes (after %nn decoding) to characters. This will never be null
      */
@@ -842,7 +771,6 @@ public class Connector extends LifecycleMBeanBase {
         } catch (UnsupportedEncodingException e) {
             log.error(sm.getString("coyoteConnector.invalidEncoding", URIEncoding, uriCharset.name()), e);
         }
-        setProperty("uRIEncoding", URIEncoding);
     }
 
 
@@ -962,11 +890,7 @@ public class Connector extends LifecycleMBeanBase {
      * @return a new Servlet request object
      */
     public Request createRequest() {
-
-        Request request = new Request();
-        request.setConnector(this);
-        return (request);
-
+        return new Request(this);
     }
 
 
@@ -977,11 +901,12 @@ public class Connector extends LifecycleMBeanBase {
      * @return a new Servlet response object
      */
     public Response createResponse() {
-
-        Response response = new Response();
-        response.setConnector(this);
-        return (response);
-
+        int size = protocolHandler.getDesiredBufferSize();
+        if (size > 0) {
+            return new Response(size);
+        } else {
+            return new Response();
+        }
     }
 
 
@@ -991,23 +916,30 @@ public class Connector extends LifecycleMBeanBase {
 
         StringBuilder sb = new StringBuilder("type=");
         sb.append(type);
-        sb.append(",port=");
-        int port = getPortWithOffset();
-        if (port > 0) {
-            sb.append(port);
+        String id = (protocolHandler != null) ? protocolHandler.getId() : null;
+        if (id != null) {
+            // Maintain MBean name compatibility, even if not accurate
+            sb.append(",port=0,address=");
+            sb.append(ObjectName.quote(id));
         } else {
-            sb.append("auto-");
-            sb.append(getProperty("nameIndex"));
-        }
-        String address = "";
-        if (addressObj instanceof InetAddress) {
-            address = ((InetAddress) addressObj).getHostAddress();
-        } else if (addressObj != null) {
-            address = addressObj.toString();
-        }
-        if (address.length() > 0) {
-            sb.append(",address=");
-            sb.append(ObjectName.quote(address));
+            sb.append(",port=");
+            int port = getPortWithOffset();
+            if (port > 0) {
+                sb.append(port);
+            } else {
+                sb.append("auto-");
+                sb.append(getProperty("nameIndex"));
+            }
+            String address = "";
+            if (addressObj instanceof InetAddress) {
+                address = ((InetAddress) addressObj).getHostAddress();
+            } else if (addressObj != null) {
+                address = addressObj.toString();
+            }
+            if (address.length() > 0) {
+                sb.append(",address=");
+                sb.append(ObjectName.quote(address));
+            }
         }
         return sb.toString();
     }
@@ -1018,7 +950,9 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void pause() {
         try {
-            protocolHandler.pause();
+            if (protocolHandler != null) {
+                protocolHandler.pause();
+            }
         } catch (Exception e) {
             log.error(sm.getString("coyoteConnector.protocolHandlerPauseFailed"), e);
         }
@@ -1030,7 +964,9 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void resume() {
         try {
-            protocolHandler.resume();
+            if (protocolHandler != null) {
+                protocolHandler.resume();
+            }
         } catch (Exception e) {
             log.error(sm.getString("coyoteConnector.protocolHandlerResumeFailed"), e);
         }
@@ -1043,6 +979,10 @@ public class Connector extends LifecycleMBeanBase {
 
         super.initInternal();
 
+        if (protocolHandler == null) {
+            throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerInstantiationFailed"));
+        }
+
         // Initialize adapter
         adapter = new CoyoteAdapter(this);
         protocolHandler.setAdapter(adapter);
@@ -1052,20 +992,32 @@ public class Connector extends LifecycleMBeanBase {
             setParseBodyMethods(getParseBodyMethods());
         }
 
-        if (protocolHandler.isAprRequired() && !AprLifecycleListener.isInstanceCreated()) {
+        if (protocolHandler.isAprRequired() && !AprStatus.isInstanceCreated()) {
             throw new LifecycleException(
                     sm.getString("coyoteConnector.protocolHandlerNoAprListener", getProtocolHandlerClassName()));
         }
-        if (protocolHandler.isAprRequired() && !AprLifecycleListener.isAprAvailable()) {
+        if (protocolHandler.isAprRequired() && !AprStatus.isAprAvailable()) {
             throw new LifecycleException(
                     sm.getString("coyoteConnector.protocolHandlerNoAprLibrary", getProtocolHandlerClassName()));
         }
-        if (AprLifecycleListener.isAprAvailable() && AprLifecycleListener.getUseOpenSSL() &&
-                protocolHandler instanceof AbstractHttp11JsseProtocol) {
-            AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
-            if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
-                // OpenSSL is compatible with the JSSE configuration, so use it if APR is available
-                jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
+        if (!protocolHandler.isAprRequired()) {
+            if (JreCompat.isJre22Available() && OpenSSLStatus.getUseOpenSSL() && OpenSSLStatus.isAvailable() &&
+                    protocolHandler instanceof AbstractHttp11Protocol) {
+                // Use FFM and OpenSSL if available
+                AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
+                if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
+                    // OpenSSL is compatible with the JSSE configuration, so use it if it is available
+                    jsseProtocolHandler.setSslImplementationName(
+                            "org.apache.tomcat.util.net.openssl.panama.OpenSSLImplementation");
+                }
+            } else if (AprStatus.isAprAvailable() && AprStatus.getUseOpenSSL() &&
+                    protocolHandler instanceof AbstractHttp11Protocol) {
+                // Use tomcat-native and OpenSSL otherwise, if available
+                AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
+                if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
+                    // OpenSSL is compatible with the JSSE configuration, so use it if APR is available
+                    jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
+                }
             }
         }
 
@@ -1086,16 +1038,23 @@ public class Connector extends LifecycleMBeanBase {
     protected void startInternal() throws LifecycleException {
 
         // Validate settings before starting
-        if (getPortWithOffset() < 0) {
+        String id = (protocolHandler != null) ? protocolHandler.getId() : null;
+        if (id == null && getPortWithOffset() < 0) {
             throw new LifecycleException(
                     sm.getString("coyoteConnector.invalidPort", Integer.valueOf(getPortWithOffset())));
         }
 
         setState(LifecycleState.STARTING);
 
+        // Configure the utility executor before starting the protocol handler
+        if (protocolHandler != null && service != null) {
+            protocolHandler.setUtilityExecutor(service.getServer().getUtilityExecutor());
+        }
+
         try {
             protocolHandler.start();
         } catch (Exception e) {
+            // Includes NPE - protocolHandler will be null for invalid protocol if throwOnFailure is false
             throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerStartFailed"), e);
         }
     }
@@ -1112,17 +1071,26 @@ public class Connector extends LifecycleMBeanBase {
         setState(LifecycleState.STOPPING);
 
         try {
-            protocolHandler.stop();
+            if (protocolHandler != null) {
+                protocolHandler.stop();
+            }
         } catch (Exception e) {
             throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerStopFailed"), e);
         }
+
+        // Remove the utility executor once the protocol handler has been stopped
+        if (protocolHandler != null) {
+            protocolHandler.setUtilityExecutor(null);
+        }
     }
 
 
     @Override
     protected void destroyInternal() throws LifecycleException {
         try {
-            protocolHandler.destroy();
+            if (protocolHandler != null) {
+                protocolHandler.destroy();
+            }
         } catch (Exception e) {
             throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerDestroyFailed"), e);
         }
@@ -1135,22 +1103,28 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    /**
-     * Provide a useful toString() implementation as it may be used when logging Lifecycle errors to identify the
-     * component.
-     */
     @Override
     public String toString() {
         // Not worth caching this right now
         StringBuilder sb = new StringBuilder("Connector[");
-        sb.append(getProtocol());
-        sb.append('-');
-        int port = getPortWithOffset();
-        if (port > 0) {
-            sb.append(port);
+        String name = (String) getProperty("name");
+        if (name == null) {
+            sb.append(getProtocol());
+            sb.append('-');
+            String id = (protocolHandler != null) ? protocolHandler.getId() : null;
+            if (id != null) {
+                sb.append(id);
+            } else {
+                int port = getPortWithOffset();
+                if (port > 0) {
+                    sb.append(port);
+                } else {
+                    sb.append("auto-");
+                    sb.append(getProperty("nameIndex"));
+                }
+            }
         } else {
-            sb.append("auto-");
-            sb.append(getProperty("nameIndex"));
+            sb.append(name);
         }
         sb.append(']');
         return sb.toString();
