@@ -125,20 +125,12 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
     // ----------------------------------------------- ContainerServlet Methods
 
 
-    /**
-     * Return the Wrapper with which we are associated.
-     */
     @Override
     public Wrapper getWrapper() {
         return this.wrapper;
     }
 
 
-    /**
-     * Set the Wrapper with which we are associated.
-     *
-     * @param wrapper The new wrapper
-     */
     @Override
     public void setWrapper(Wrapper wrapper) {
 
@@ -158,9 +150,6 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
     // --------------------------------------------------------- Public Methods
 
 
-    /**
-     * Finalize this servlet.
-     */
     @Override
     public void destroy() {
 
@@ -169,15 +158,6 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
     }
 
 
-    /**
-     * Process a GET request for the specified resource.
-     *
-     * @param request  The servlet request we are processing
-     * @param response The servlet response we are creating
-     *
-     * @exception IOException      if an input/output error occurs
-     * @exception ServletException if a servlet-specified error occurs
-     */
     @Override
     public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
 
@@ -455,7 +435,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
             Container child = engine.findChild(name);
             engine.removeChild(child);
             if (child instanceof ContainerBase) {
-                ((ContainerBase) child).destroy();
+                child.destroy();
             }
         } catch (Exception e) {
             writer.println(smClient.getString("hostManagerServlet.exception", e.toString()));
@@ -612,7 +592,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
 
         try {
             MBeanServer platformMBeanServer = ManagementFactory.getPlatformMBeanServer();
-            ObjectName oname = new ObjectName(engine.getDomain() + ":type=StoreConfig");
+            ObjectName oname = new ObjectName("Catalina:type=StoreConfig");
             platformMBeanServer.invoke(oname, "storeConfig", null, null);
             writer.println(smClient.getString("hostManagerServlet.persisted"));
         } catch (Exception e) {
@@ -621,7 +601,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
             // catch InstanceNotFoundException when StoreConfig is not enabled instead of printing
             // the failure message
             if (e instanceof InstanceNotFoundException) {
-                writer.println("Please enable StoreConfig to use this feature.");
+                writer.println(smClient.getString("hostManagerServlet.noStoreConfig"));
             } else {
                 writer.println(smClient.getString("hostManagerServlet.exception", e.toString()));
             }
