@@ -36,7 +36,6 @@ import org.apache.catalina.User;
 import org.apache.catalina.UserDatabase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.modeler.Registry;
 import org.apache.tomcat.util.res.StringManager;
 
 
@@ -64,12 +63,6 @@ public class GlobalResourcesLifecycleListener implements LifecycleListener {
     protected Lifecycle component = null;
 
 
-    /**
-     * The configuration information registry for our managed beans.
-     */
-    protected static final Registry registry = MBeanUtils.createRegistry();
-
-
     // ---------------------------------------------- LifecycleListener Methods
 
     /**
@@ -179,6 +172,11 @@ public class GlobalResourcesLifecycleListener implements LifecycleListener {
             throw new IllegalArgumentException(sm.getString("globalResources.createError.userDatabase", name), e);
         }
 
+        if (database.isSparse()) {
+            // Avoid loading all the database as mbeans
+            return;
+        }
+
         // Create the MBeans for each defined Role
         Iterator<Role> roles = database.getRoles();
         while (roles.hasNext()) {
