@@ -21,9 +21,12 @@ import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.util.Hashtable;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.util.security.PermissionCheck;
 
 /**
@@ -32,6 +35,7 @@ import org.apache.tomcat.util.security.PermissionCheck;
 public final class IntrospectionUtils {
 
     private static final Log log = LogFactory.getLog(IntrospectionUtils.class);
+    private static final StringManager sm = StringManager.getManager(IntrospectionUtils.class);
 
     /**
      * Find a method with the right name If found, call the method ( if param is
@@ -43,17 +47,26 @@ public final class IntrospectionUtils {
      * @return <code>true</code> if operation was successful
      */
     public static boolean setProperty(Object o, String name, String value) {
-        return setProperty(o,name,value,true);
+        return setProperty(o, name, value, true, null);
+    }
+
+    public static boolean setProperty(Object o, String name, String value,
+            boolean invokeSetProperty) {
+        return setProperty(o, name, value, invokeSetProperty, null);
     }
 
     @SuppressWarnings("null") // setPropertyMethodVoid is not null when used
     public static boolean setProperty(Object o, String name, String value,
-            boolean invokeSetProperty) {
-        if (log.isDebugEnabled()) {
-            log.debug("IntrospectionUtils: setProperty(" +
+            boolean invokeSetProperty, StringBuilder actualMethod) {
+        if (log.isTraceEnabled()) {
+            log.trace("IntrospectionUtils: setProperty(" +
                     o.getClass() + " " + name + "=" + value + ")");
         }
 
+        if (actualMethod == null && XReflectionIntrospectionUtils.isEnabled()) {
+            return XReflectionIntrospectionUtils.setPropertyInternal(o, name, value, invokeSetProperty);
+        }
+
         String setter = "set" + capitalize(name);
 
         try {
@@ -66,8 +79,10 @@ public final class IntrospectionUtils {
                 Class<?> paramT[] = item.getParameterTypes();
                 if (setter.equals(item.getName()) && paramT.length == 1
                         && "java.lang.String".equals(paramT[0].getName())) {
-
                     item.invoke(o, new Object[]{value});
+                    if (actualMethod != null) {
+                        actualMethod.append(item.getName()).append("(\"").append(escape(value)).append("\")");
+                    }
                     return true;
                 }
             }
@@ -90,6 +105,13 @@ public final class IntrospectionUtils {
                         } catch (NumberFormatException ex) {
                             ok = false;
                         }
+                        if (actualMethod != null) {
+                            if ("java.lang.Integer".equals(paramType.getName())) {
+                                actualMethod.append(method.getName()).append("(Integer.valueOf(\"").append(value).append("\"))");
+                            } else {
+                                actualMethod.append(method.getName()).append("(Integer.parseInt(\"").append(value).append("\"))");
+                            }
+                        }
                         // Try a setFoo ( long )
                     } else if ("java.lang.Long".equals(paramType.getName())
                             || "long".equals(paramType.getName())) {
@@ -98,12 +120,24 @@ public final class IntrospectionUtils {
                         } catch (NumberFormatException ex) {
                             ok = false;
                         }
-
+                        if (actualMethod != null) {
+                            if ("java.lang.Long".equals(paramType.getName())) {
+                                actualMethod.append(method.getName()).append("(Long.valueOf(\"").append(value).append("\"))");
+                            } else {
+                                actualMethod.append(method.getName()).append("(Long.parseLong(\"").append(value).append("\"))");
+                            }
+                        }
                         // Try a setFoo ( boolean )
                     } else if ("java.lang.Boolean".equals(paramType.getName())
                             || "boolean".equals(paramType.getName())) {
                         params[0] = Boolean.valueOf(value);
-
+                        if (actualMethod != null) {
+                            if ("java.lang.Boolean".equals(paramType.getName())) {
+                                actualMethod.append(method.getName()).append("(Boolean.valueOf(\"").append(value).append("\"))");
+                            } else {
+                                actualMethod.append(method.getName()).append("(Boolean.parseBoolean(\"").append(value).append("\"))");
+                            }
+                        }
                         // Try a setFoo ( InetAddress )
                     } else if ("java.net.InetAddress".equals(paramType
                             .getName())) {
@@ -111,15 +145,17 @@ public final class IntrospectionUtils {
                             params[0] = InetAddress.getByName(value);
                         } catch (UnknownHostException exc) {
                             if (log.isDebugEnabled()) {
-                                log.debug("IntrospectionUtils: Unable to resolve host name:" + value);
+                                log.debug(sm.getString("introspectionUtils.hostResolutionFail", value));
                             }
                             ok = false;
                         }
-
+                        if (actualMethod != null) {
+                            actualMethod.append(method.getName()).append("(InetAddress.getByName(\"").append(value).append("\"))");
+                        }
                         // Unknown type
                     } else {
-                        if (log.isDebugEnabled()) {
-                            log.debug("IntrospectionUtils: Unknown type " +
+                        if (log.isTraceEnabled()) {
+                            log.trace("IntrospectionUtils: Unknown type " +
                                     paramType.getName());
                         }
                     }
@@ -144,6 +180,9 @@ public final class IntrospectionUtils {
             // Ok, no setXXX found, try a setProperty("name", "value")
             if (invokeSetProperty && (setPropertyMethodBool != null ||
                     setPropertyMethodVoid != null)) {
+                if (actualMethod != null) {
+                    actualMethod.append("setProperty(\"").append(name).append("\", \"").append(escape(value)).append("\")");
+                }
                 Object params[] = new Object[2];
                 params[0] = name;
                 params[1] = value;
@@ -167,23 +206,48 @@ public final class IntrospectionUtils {
                 }
             }
 
-        } catch (IllegalArgumentException ex2) {
-            log.warn("IAE " + o + " " + name + " " + value, ex2);
-        } catch (SecurityException ex1) {
-            log.warn("IntrospectionUtils: SecurityException for " +
-                    o.getClass() + " " + name + "=" + value + ")", ex1);
-        } catch (IllegalAccessException iae) {
-            log.warn("IntrospectionUtils: IllegalAccessException for " +
-                    o.getClass() + " " + name + "=" + value + ")", iae);
-        } catch (InvocationTargetException ie) {
-            ExceptionUtils.handleThrowable(ie.getCause());
-            log.warn("IntrospectionUtils: InvocationTargetException for " +
-                    o.getClass() + " " + name + "=" + value + ")", ie);
+        } catch (IllegalArgumentException | SecurityException | IllegalAccessException e) {
+            log.warn(sm.getString("introspectionUtils.setPropertyError", name, value, o.getClass()), e);
+        } catch (InvocationTargetException e) {
+            ExceptionUtils.handleThrowable(e.getCause());
+            log.warn(sm.getString("introspectionUtils.setPropertyError", name, value, o.getClass()), e);
         }
         return false;
     }
 
+    /**
+     * @param s
+     *            the input string
+     * @return escaped string, per Java rule
+     */
+    public static String escape(String s) {
+
+        if (s == null) {
+            return "";
+        }
+
+        StringBuilder b = new StringBuilder();
+        for (int i = 0; i < s.length(); i++) {
+            char c = s.charAt(i);
+            if (c == '"') {
+                b.append('\\').append('"');
+            } else if (c == '\\') {
+                b.append('\\').append('\\');
+            } else if (c == '\n') {
+                b.append('\\').append('n');
+            } else if (c == '\r') {
+                b.append('\\').append('r');
+            } else {
+                b.append(c);
+            }
+        }
+        return b.toString();
+    }
+
     public static Object getProperty(Object o, String name) {
+        if (XReflectionIntrospectionUtils.isEnabled()) {
+            return XReflectionIntrospectionUtils.getPropertyInternal(o, name);
+        }
         String getter = "get" + capitalize(name);
         String isGetter = "is" + capitalize(name);
 
@@ -213,22 +277,15 @@ public final class IntrospectionUtils {
                 return getPropertyMethod.invoke(o, params);
             }
 
-        } catch (IllegalArgumentException ex2) {
-            log.warn("IAE " + o + " " + name, ex2);
-        } catch (SecurityException ex1) {
-            log.warn("IntrospectionUtils: SecurityException for " +
-                    o.getClass() + " " + name + ")", ex1);
-        } catch (IllegalAccessException iae) {
-            log.warn("IntrospectionUtils: IllegalAccessException for " +
-                    o.getClass() + " " + name + ")", iae);
-        } catch (InvocationTargetException ie) {
-            if (ie.getCause() instanceof NullPointerException) {
+        } catch (IllegalArgumentException | SecurityException | IllegalAccessException e) {
+            log.warn(sm.getString("introspectionUtils.getPropertyError", name, o.getClass()), e);
+        } catch (InvocationTargetException e) {
+            if (e.getCause() instanceof NullPointerException) {
                 // Assume the underlying object uses a storage to represent an unset property
                 return null;
             }
-            ExceptionUtils.handleThrowable(ie.getCause());
-            log.warn("IntrospectionUtils: InvocationTargetException for " +
-                    o.getClass() + " " + name + ")", ie);
+            ExceptionUtils.handleThrowable(e.getCause());
+            log.warn(sm.getString("introspectionUtils.getPropertyError", name, o.getClass()), e);
         }
         return null;
     }
@@ -359,7 +416,7 @@ public final class IntrospectionUtils {
         objectMethods.clear();
     }
 
-    private static final Hashtable<Class<?>,Method[]> objectMethods = new Hashtable<>();
+    private static final Map<Class<?>,Method[]> objectMethods = new ConcurrentHashMap<>();
 
     public static Method[] findMethods(Class<?> c) {
         Method methods[] = objectMethods.get(c);
@@ -372,15 +429,18 @@ public final class IntrospectionUtils {
         return methods;
     }
 
-    @SuppressWarnings("null") // params cannot be null when comparing lengths
     public static Method findMethod(Class<?> c, String name,
             Class<?> params[]) {
         Method methods[] = findMethods(c);
         for (Method method : methods) {
             if (method.getName().equals(name)) {
                 Class<?> methodParams[] = method.getParameterTypes();
-                if (params == null && methodParams.length == 0) {
-                    return method;
+                if (params == null) {
+                    if (methodParams.length == 0) {
+                        return method;
+                    } else {
+                        continue;
+                    }
                 }
                 if (params.length != methodParams.length) {
                     continue;
@@ -402,13 +462,11 @@ public final class IntrospectionUtils {
 
     public static Object callMethod1(Object target, String methodN,
             Object param1, String typeParam1, ClassLoader cl) throws Exception {
-        if (target == null || param1 == null) {
-            throw new IllegalArgumentException(
-                    "IntrospectionUtils: Assert: Illegal params " +
-                    target + " " + param1);
+        if (target == null || methodN == null || param1 == null) {
+            throw new IllegalArgumentException(sm.getString("introspectionUtils.nullParameter"));
         }
-        if (log.isDebugEnabled()) {
-            log.debug("IntrospectionUtils: callMethod1 " +
+        if (log.isTraceEnabled()) {
+            log.trace("IntrospectionUtils: callMethod1 " +
                     target.getClass().getName() + " " +
                     param1.getClass().getName() + " " + typeParam1);
         }
@@ -421,8 +479,7 @@ public final class IntrospectionUtils {
         }
         Method m = findMethod(target.getClass(), methodN, params);
         if (m == null) {
-            throw new NoSuchMethodException(target.getClass().getName() + " "
-                    + methodN);
+            throw new NoSuchMethodException(sm.getString("introspectionUtils.noMethod", methodN, target, target.getClass()));
         }
         try {
             return m.invoke(target, new Object[] { param1 });
@@ -438,27 +495,24 @@ public final class IntrospectionUtils {
         m = findMethod(target.getClass(), methodN, typeParams);
         if (m == null) {
             if (log.isDebugEnabled()) {
-                log.debug("IntrospectionUtils: Can't find method " + methodN +
-                        " in " + target + " CLASS " + target.getClass());
+                log.debug(sm.getString("introspectionUtils.noMethod", methodN, target, target.getClass()));
             }
             return null;
         }
         try {
             Object o = m.invoke(target, params);
 
-            if (log.isDebugEnabled()) {
-                // debug
+            if (log.isTraceEnabled()) {
                 StringBuilder sb = new StringBuilder();
-                sb.append(target.getClass().getName()).append('.')
-                        .append(methodN).append("( ");
+                sb.append(target.getClass().getName()).append('.').append(methodN).append('(');
                 for (int i = 0; i < params.length; i++) {
                     if (i > 0) {
                         sb.append(", ");
                     }
                     sb.append(params[i]);
                 }
-                sb.append(")");
-                log.debug("IntrospectionUtils:" + sb.toString());
+                sb.append(')');
+                log.trace("IntrospectionUtils:" + sb.toString());
             }
             return o;
         } catch (InvocationTargetException ie) {
@@ -489,20 +543,19 @@ public final class IntrospectionUtils {
                 result = InetAddress.getByName(object);
             } catch (UnknownHostException exc) {
                 if (log.isDebugEnabled()) {
-                    log.debug("IntrospectionUtils: Unable to resolve host name:" +
-                            object);
+                    log.debug(sm.getString("introspectionUtils.hostResolutionFail", object));
                 }
             }
 
             // Unknown type
         } else {
-            if (log.isDebugEnabled()) {
-                log.debug("IntrospectionUtils: Unknown type " +
+            if (log.isTraceEnabled()) {
+                log.trace("IntrospectionUtils: Unknown type " +
                         paramType.getName());
             }
         }
         if (result == null) {
-            throw new IllegalArgumentException("Can't convert argument: " + object);
+            throw new IllegalArgumentException(sm.getString("introspectionUtils.conversionError", object, paramType.getName()));
         }
         return result;
     }
@@ -549,12 +602,12 @@ public final class IntrospectionUtils {
     // -------------------- Get property --------------------
     // This provides a layer of abstraction
 
-    public static interface PropertySource {
-        public String getProperty(String key);
+    public interface PropertySource {
+        String getProperty(String key);
     }
 
 
-    public static interface SecurePropertySource extends PropertySource {
+    public interface SecurePropertySource extends PropertySource {
 
         /**
          * Obtain a property value, checking that code associated with the
@@ -571,6 +624,6 @@ public final class IntrospectionUtils {
          *         or if {@link PermissionCheck#check(java.security.Permission)}
          *         fails
          */
-        public String getProperty(String key, ClassLoader classLoader);
+        String getProperty(String key, ClassLoader classLoader);
     }
 }
