@@ -20,9 +20,11 @@ import java.beans.BeanInfo;
 import java.beans.Introspector;
 import java.beans.PropertyDescriptor;
 import java.beans.PropertyEditorSupport;
+import java.io.File;
 import java.io.IOException;
 import java.nio.charset.CodingErrorAction;
 import java.util.Date;
+import java.util.Scanner;
 
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.jsp.JspException;
@@ -40,8 +42,12 @@ import javax.servlet.jsp.tagext.VariableInfo;
 import org.junit.Assert;
 import org.junit.Test;
 
+import org.apache.catalina.Context;
 import org.apache.catalina.LifecycleException;
+import org.apache.catalina.Wrapper;
+import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
+import org.apache.jasper.servlet.JasperInitializer;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.compat.JreCompat;
 
@@ -267,6 +273,63 @@ public class TestGenerator extends TomcatBaseTest {
         Assert.assertTrue(result.endsWith("999 Hello world!\n"));
     }
 
+
+    // https://bz.apache.org/bugzilla/show_bug.cgi?id=43400
+    @Test
+    public void testTagsWithEnums() throws Exception {
+        getTomcatInstanceTestWebapp(false, true);
+
+        ByteChunk res = getUrl("http://localhost:" + getPort() + "/test/bug43nnn/bug43400.jsp");
+
+        String result = res.toString();
+        System.out.println(result);
+        assertEcho(result, "ASYNC");
+    }
+
+    @Test
+    public void testTrimSpacesExtended01() throws Exception {
+        doTestTrimSpacesExtended(false);
+    }
+
+    @Test
+    public void testTrimSpacesExtended02() throws Exception {
+        doTestTrimSpacesExtended(true);
+    }
+
+    private void doTestTrimSpacesExtended(boolean removeBlankLines) throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        ctxt.addServletContainerInitializer(new JasperInitializer(), null);
+
+        Tomcat.initWebappDefaults(ctxt);
+        Wrapper w = (Wrapper) ctxt.findChild("jsp");
+        if (removeBlankLines) {
+            w.addInitParameter("trimSpaces", "extended");
+        }
+
+        tomcat.start();
+
+        ByteChunk res = getUrl("http://localhost:" + getPort() + "/jsp/trim-spaces-extended.jsp");
+
+        String result = res.toString();
+        Scanner scanner = new Scanner(result);
+        int blankLineCount = 0;
+        while (scanner.hasNextLine()) {
+            String line = scanner.nextLine();
+            if (line.length() == 0) {
+                blankLineCount++;
+            }
+        }
+        if (!removeBlankLines && blankLineCount == 0) {
+            Assert.fail("TrimSpaceOptions.EXTENDED not configured but balnk lines have been removed");
+        } else if (removeBlankLines && blankLineCount > 0) {
+            Assert.fail("TrimSpaceOptions.EXTENDED does not allow the line to be just a new line character");
+        }
+        scanner.close();
+    }
+
     @Test
     public void testEscape01() {
         String result = Generator.escape("\"\\\n\r");
@@ -463,6 +526,25 @@ public class TestGenerator extends TomcatBaseTest {
         }
     }
 
+    private static boolean tagTesterTagReleaseReleased = false;
+
+    public static class TesterTagRelease extends TesterTag {
+        private String data;
+
+        public String getData() {
+            return data;
+        }
+
+        public void setData(String data) {
+            this.data = data;
+        }
+
+        @Override
+        public void release() {
+            tagTesterTagReleaseReleased = true;
+        }
+    }
+
     public static class DataPropertyEditor extends PropertyEditorSupport {
     }
 
@@ -884,6 +966,38 @@ public class TestGenerator extends TomcatBaseTest {
         Assert.assertEquals(body.toString(), HttpServletResponse.SC_OK, rc);
     }
 
+    @Test
+    public void testTagReleaseWithPooling() throws Exception {
+        doTestTagRelease(true);
+    }
+
+    @Test
+    public void testTagReleaseWithoutPooling() throws Exception {
+        doTestTagRelease(false);
+    }
+
+    public void doTestTagRelease(boolean enablePooling) throws Exception {
+        tagTesterTagReleaseReleased = false;
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        ctxt.addServletContainerInitializer(new JasperInitializer(), null);
+
+        Tomcat.initWebappDefaults(ctxt);
+        Wrapper w = (Wrapper) ctxt.findChild("jsp");
+        w.addInitParameter("enablePooling", String.valueOf(enablePooling));
+
+        tomcat.start();
+
+        getUrl("http://localhost:" + getPort() + "/jsp/generator/release.jsp");
+        if (enablePooling) {
+            Assert.assertFalse(tagTesterTagReleaseReleased);
+        } else {
+            Assert.assertTrue(tagTesterTagReleaseReleased);
+        }
+    }
+
     private void doTestJsp(String jspName) throws Exception {
         doTestJsp(jspName, HttpServletResponse.SC_OK);
     }
